@@ -2,5 +2,5 @@ from django.apps import AppConfig
 
 
 class DBMIClientConfig(AppConfig):
-    name = 'dbmi_client'
-    verbose_name = 'DBMI Client'
+    name = "dbmi_client"
+    verbose_name = "DBMI Client"
