@@ -9,6 +9,7 @@ from dbmi_client.settings import dbmi_settings
 
 # Get the app logger
 import logging
+
 logger = logging.getLogger(dbmi_settings.LOGGER_NAME)
 
 
@@ -19,15 +20,15 @@ def create_dbmi_user(request, **profile):
     email = authn.get_jwt_email(request, verify=False)
 
     # Update kwargs
-    profile['email'] = email
+    profile["email"] = email
 
     # Build the URL (needs trailing slash)
     url = furl(dbmi_settings.REG_URL)
-    url.path.segments.extend(['api', 'register', ''])
+    url.path.segments.extend(["api", "register", ""])
 
     response = requests.post(url.url, headers=authn.dbmi_http_headers(request), data=json.dumps(profile))
     if not response.ok:
-        logger.error('Create user response: {}'.format(response.content))
+        logger.error("Create user response: {}".format(response.content))
 
     return response.json()
 
@@ -41,18 +42,19 @@ def get_dbmi_user(request, email=None):
 
     # Build the URL (needs trailing slash)
     url = furl(dbmi_settings.REG_URL)
-    url.path.segments.extend(['api', 'register', ''])
+    url.path.segments.extend(["api", "register", ""])
 
     # Add email
-    url.query.params.add('email', email)
+    url.query.params.add("email", email)
 
     # Requests for profiles are limited to the profile for the requesting user
     response = requests.get(url.url, headers=authn.dbmi_http_headers(request))
     if not response.ok:
-        logger.error('Get user response: {}'.format(response.content))
+        logger.error("Get user response: {}".format(response.content))
 
     # Return the profile
-    return response['results'][0] if len(response['results']) else None
+    profiles = response.json()["results"]
+    return next(iter(profiles), None)
 
 
 def update_dbmi_user(request, **profile):
@@ -66,8 +68,8 @@ def update_dbmi_user(request, **profile):
     if not reg_profile:
 
         # Ensure email is in their profile
-        if 'email' not in profile:
-            profile['email'] = email
+        if "email" not in profile:
+            profile["email"] = email
 
         # Create the profile
         return create_dbmi_user(request, **profile)
@@ -75,11 +77,11 @@ def update_dbmi_user(request, **profile):
     else:
         # Build the URL (needs trailing slash)
         url = furl(dbmi_settings.REG_URL)
-        url.path.segments.extend(['api', 'register', reg_profile['id'], ''])
+        url.path.segments.extend(["api", "register", reg_profile["id"], ""])
 
         response = requests.put(url.url, headers=authn.dbmi_http_headers(request), data=json.dumps(profile))
         if not response.ok:
-            logger.error('Update user response: {}'.format(response.content))
+            logger.error("Update user response: {}".format(response.content))
 
         return response.json()
 
@@ -89,30 +91,30 @@ def send_email_confirmation(request, success_url, title=None, icon=None, subject
 
     # Build the URL (needs trailing slash)
     url = furl(dbmi_settings.REG_URL)
-    url.path.segments.extend(['api', 'register', 'send_confirmation_email', ''])
+    url.path.segments.extend(["api", "register", "send_confirmation_email", ""])
 
     # Add extra data to define look and feel of email, if passed
     branding = {}
     if title:
-        branding['title'] = title
+        branding["title"] = title
     if icon:
-        branding['icon'] = icon
+        branding["icon"] = icon
     if subject:
-        branding['subject'] = subject
+        branding["subject"] = subject
 
     # Set data for request
     data = {
-        'success_url': success_url,
+        "success_url": success_url,
     }
 
     # Check for branding
     if branding:
-        data['branding'] = base64.b64encode(json.dumps(branding).encode()).decode()
+        data["branding"] = base64.b64encode(json.dumps(branding).encode()).decode()
 
     # Make the call
     response = requests.post(url.url, headers=authn.dbmi_http_headers(request), data=json.dumps(data))
     if not response.ok:
-        logger.error('Confirmation email response: {}'.format(response.content))
+        logger.error("Confirmation email response: {}".format(response.content))
 
     return response.ok
 
@@ -122,25 +124,23 @@ def check_email_confirmation(request):
 
     # Build the URL (needs trailing slash)
     url = furl(dbmi_settings.REG_URL)
-    url.path.segments.extend(['api', 'register', ''])
-    url.query.params.add('email', authn.get_jwt_email(request, verify=False))
+    url.path.segments.extend(["api", "register", ""])
+    url.query.params.add("email", authn.get_jwt_email(request, verify=False))
 
     # Make the call
     response = requests.get(url.url, headers=authn.dbmi_http_headers(request))
     if not response.ok:
-        logger.error('Confirmation email response: {}'.format(response.content))
+        logger.error("Confirmation email response: {}".format(response.content))
         return None
 
     try:
         # Parse the profile for the status
-        email_status = response.json()['results'][0]['email_confirmed']
-        logger.debug('Email confirmation status: {}'.format(email_status))
+        email_status = response.json()["results"][0]["email_confirmed"]
+        logger.debug("Email confirmation status: {}".format(email_status))
 
         return email_status
 
     except (KeyError, IndexError) as e:
-        logger.error('Failed parsing profile: {}'.format(e))
+        logger.error("Failed parsing profile: {}".format(e))
 
     return None
-
-
